@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Warp Networks S.L.
+# Copyright (C) 2006-2007 Warp Networks S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -98,6 +98,8 @@ if ( defined $ENV{OPENSSL} ) {
   $ENV{OPENSSL} = $openssl;
 }
 
+# Group: Public methods
+
 # Constructor: _create
 #
 #      Create a new EBox::CA object
@@ -184,14 +186,16 @@ sub isCreated
 #
 # Parameters:
 #
-#       countryName  - country name {2 letter code} (eg, ES) (Optional)
-#       stateName     - state or province name (eg, Aragon) (Optional)
-#       localityName  - locality name (eg, Zaragoza) (Optional)
+#       countryName  - country name {2 letter code} (eg, ES) *(Optional)*
+#       stateName     - state or province name (eg, Aragon) *(Optional)*
+#       localityName  - locality name (eg, Zaragoza) *(Optional)*
 #       orgName       - organization name (eg, company name)
-#       orgNameUnit  - organizational unit name (eg, section name) (Optional)
-#       commonName    - common name from the CA (Optional)
-#       caKeyPassword - passphrase for generating keys (Optional)
-#       days         - expire day of self signed certificate (Optional)
+#       orgNameUnit  - organizational unit name (eg, section name) *(Optional)*
+#       commonName    - common name from the CA *(Optional)*
+#       caKeyPassword - passphrase for generating keys *(Optional)*
+#       days         - expire day of self signed certificate *(Optional)*
+#
+#       - Named parameters
 #
 # Returns:
 #
@@ -200,7 +204,7 @@ sub isCreated
 #
 # Exceptions:
 #
-#      EBox::Exceptions::DataMissing - if any required parameter is
+#      <EBox::Exceptions::DataMissing> - if any required parameter is
 #      missing
 
 sub createCA {
@@ -310,6 +314,8 @@ sub createCA {
 #	      );
 
   #unlink (CAREQ);
+  $self->_setPasswordRequired(defined($self->{caKeyPassword}));
+
   return 1;
 
 }
@@ -356,6 +362,29 @@ sub destroyCA
 
   }
 
+# Method: passwordRequired
+#
+#       Check if the password is required to sign the certificates
+#       within this Certification Authority
+#
+# Returns:
+#
+#       boolean - true if pass is required, false otherwise
+#
+sub passwordRequired
+  {
+
+      my ($self) = @_;
+
+      my $caState = $self->currentCACertificateState();
+
+      # Password could be required only if a CA has been created
+      return undef unless ($caState eq 'V');
+
+      return $self->get_bool('pass_required');
+
+  }
+
 # Method: revokeCACertificate
 #
 #       Revoke the self-signed CA certificate and subsequently all the
@@ -393,24 +422,23 @@ sub revokeCACertificate
     }
 
     # Revoked all issued and valid certificates
-    my $listCerts = $self->listCertificates();
+    # Revoked only valid ones
+    # We ensure not to revoke the CA cert before the others
+    my $listCerts = $self->listCertificates( state => 'V',
+                                             excludeCA => 1);
     foreach my $element (@{$listCerts}) {
-      # Revoked only valid ones
-      # We ensure not to revoke the CA cert before the others
-      if ($element->{state} eq 'V' and
-	  -f ( KEYSDIR . $element->{dn}->attribute('commonName') . ".pem" ) ) {
-
 	$self->revokeCertificate(commonName    => $element->{dn}->attribute('commonName'),
 				 reason        => "cessationOfOperation",
-				 caKeyPassword => $args{caKeyPassword}
+				 caKeyPassword => $args{caKeyPassword},
+                                 force         => $args{force},
 				);
-      }
     }
 
     my $retVal = $self->revokeCertificate(commonName    => "",
 					  reason        => $args{reason},
 					  caKeyPassword => $args{caKeyPassword},
-					  certFile      => CACERT);
+					  certFile      => CACERT,
+                                          force         => $args{force});
 
     $self->{caExpirationDate} = undef;
 
@@ -500,6 +528,8 @@ sub issueCACertificate
 #		"orgName=" . $self->{dn}->attribute("orgName") . ",days=" . $args{days}
 #	       );
 
+    $self->_setPasswordRequired( defined( $args{caKeyPassword} ));
+
     return $ret;
 
   }
@@ -950,6 +980,8 @@ sub revokeCertificate {
 #       excludeCA - boolean indicating whether the valid CA certificate
 #                   should be excluded in the response (Optional)
 #
+#       - Named parameters
+#
 # Returns:
 #
 #       A reference to an array containing hashes which have the following
@@ -1059,7 +1091,7 @@ sub listCertificates
 #
 # Exceptions:
 #
-#      DataMissing - if any required parameter is missing
+#      <EBox::Exceptions::DataMissing> - if any required parameter is missing
 
 sub getCertificateMetadata
   {
@@ -1277,8 +1309,7 @@ sub renewCertificate
       if (defined($args{endDate}) and not UNIVERSAL::isa($args{endDate}, "Date::Calc"));
 
     # Check the expiration date unless issuing the CA cert
-    if (not defined($args{certFile}) or
-	$args{certFile} ne CACERT) {
+    unless (defined($args{certFile}) or $args{certFile} eq CACERT) {
       my $userExpDay = undef;
       if ( defined($args{days}) ) {
 	$userExpDay = Date::Calc::Object->now() + [0, 0, +$args{days}, 0, 0, 0];
@@ -1673,6 +1704,8 @@ sub restoreConfig
 
   }
 
+# Group: Private methods
+
 # Obtain the public key given the private key 
 #
 # Return public key path if it is correct or undef if password is
@@ -2174,7 +2207,7 @@ sub _certsInUse # (cn?, isCACert?)
     foreach my $obs (@observers) {
       if( $obs->certificateRevoked($cn, $isCACert) ){
 	return 1;
-      }
+    }
     }
 
     return undef;
@@ -2196,13 +2229,21 @@ sub _freeCert # (cn?, isCACert?)
     my $global = EBox::Global->getInstance();
     my @observers = @{$global->modInstancesOfType('EBox::CA::Observer')};
     foreach my $obs (@observers) {
-      if( $obs->freeCertificate($cn) ){
-	return 1;
-      }
+        $obs->freeCertificate($cn);
     }
 
   }
 
+# Set the CA as passwordful or passwordless
+sub _setPasswordRequired # (required)
+  {
+
+    my ($self, $required) = @_;
+
+    $self->set_bool('pass_required', $required);
+
+  }
+
 # Setting logAdmin actions
 sub _setLogAdminActions
   {
